#include "StdAfx.h"
#define _USE_MATH_DEFINES
#include <math.h>
#include "PrecisionApproachScreen.h"
#include "PARSettingsDialog.h"


//---CPrecisionApproachScreen------------------------------------------

CPrecisionApproachScreen :: CPrecisionApproachScreen ( void )
{
    // default values
    m_Description = "LHBP 31R PAR" ;
    m_Latitude    = "N047.25.24.615" ;
    m_Longitude   = "E019.17.35.260" ;
    m_Altitude    = 550 ;
    m_Heading     = 310 ;
    m_Slope       = 2.5 ;
}


//---~CPrecisionApproachScreen-----------------------------------------

CPrecisionApproachScreen :: ~CPrecisionApproachScreen ( void )
{
}


//---OnAsrContentLoaded------------------------------------------------

void    CPrecisionApproachScreen :: OnAsrContentLoaded ( bool Loaded )
{
    const char  * p_value ;


    // get the new values
    if (( p_value = GetDataFromAsr ( "Description" )) != NULL )
        m_Description = p_value ;
    if (( p_value = GetDataFromAsr ( "Longitude" )) != NULL )
        m_Longitude = p_value ;
    if (( p_value = GetDataFromAsr ( "Latitude" )) != NULL )
        m_Latitude = p_value ;
    if (( p_value = GetDataFromAsr ( "Altitude" )) != NULL )
        m_Altitude = atoi ( p_value ) ;
    if (( p_value = GetDataFromAsr ( "Heading" )) != NULL )
        m_Heading = atoi ( p_value ) ;
    if (( p_value = GetDataFromAsr ( "Slope" )) != NULL )
        m_Slope = atof ( p_value ) ;

    // calculate threshold position and max slope height
    m_RwyPos.LoadFromStrings ( m_Longitude, m_Latitude ) ;
    m_MaxHeight = m_Altitude + (int) ( 20.0 * 6076.0 * sin ( m_Slope / 180.0 * M_PI )) ;
}


//---OnAsrContentToBeSaved----------------------------------------------

void    CPrecisionApproachScreen :: OnAsrContentToBeSaved ( void )
{
    CString str ;


    // simply save all current data
    SaveDataToAsr ( "Description", "PAR description", m_Description ) ;
    SaveDataToAsr ( "Longitude", "RWY treshold longitude", m_Longitude ) ;
    SaveDataToAsr ( "Latitude", "RWY treshold latitude", m_Latitude ) ;
    str.Format ( "%d", m_Altitude ) ;
    SaveDataToAsr ( "Altitude", "RWY treshold altitude", str ) ;
    str.Format ( "%d", m_Heading ) ;
    SaveDataToAsr ( "Heading", "RWY heading", str ) ;
    str.Format ( "%.1f", m_Slope ) ;
    SaveDataToAsr ( "Slope", "Glide slope angle", str ) ;
}


//---OnRefresh----------------------------------------------------------

void    CPrecisionApproachScreen :: OnRefresh ( HDC hDC,
                                                int Phase )
{
    // only in first phase
    if ( Phase != EuroScopePlugIn :: REFRESH_PHASE_BEFORE_TAGS )
        return ;

    // I am using MFC, so load the DC in to MFC structure
    CDC     dc ;
    dc.Attach ( hDC ) ;

    // pens
    CPen    pen1 ( 0, 5, RGB ( 128, 128, 128 )) ;
    CPen    pen2 ( 0, 0, RGB ( 128, 128, 128 )) ;
    CPen    pen3 ( 0, 3, RGB ( 255, 255, 255 )) ;
    CPen    pen4 ( 0, 5, RGB ( 255,   0,   0 )) ;
    CPen    * p_old_pen ;

    // get the area that is not coevered by the toolbar and the chat
    CRect   rect1 = GetRadarArea () ;
    CRect   rect2 = GetChatArea () ;
    rect1.bottom = rect2.top ;
    int     middley = rect1.CenterPoint ().y ;
    int     lovermiddley = ( middley + rect1.bottom ) / 2 ;
    int     width = rect1.Width () - 10 ;
    int     height = middley - 20 - ( rect1.top - 5 ) ;

    // separate the top and the bottom half by a thin line
    p_old_pen = dc.SelectObject ( & pen1 ) ;
    dc.MoveTo ( rect1.left, middley ) ;
    dc.LineTo ( rect1.right, middley ) ;

    // glide slope on the top
    dc.SelectObject ( & pen2 ) ;
    dc.MoveTo ( rect1.left + 5, middley - 20 ) ;
    dc.LineTo ( rect1.right - 5, rect1.top - 5 ) ;

    // extended centerline
    dc.MoveTo ( rect1.left + 5, lovermiddley ) ;
    dc.LineTo ( rect1.right - 5, lovermiddley ) ;

    // 1 NM ticks
    for ( int i = 0 ; i <= 20 ; i ++ )
        {
        if ( i % 5 == 0 )
            dc.SelectObject ( & pen1 ) ;
        else
            dc.SelectObject ( & pen2 ) ;

        dc.MoveTo ( rect1.left + 5 + i * width / 20, middley - 10 ) ;
        dc.LineTo ( rect1.left + 5 + i * width / 20, middley + 10 ) ;
        }

    // 1000 feet ticks
    dc.SelectObject ( & pen2 ) ;
    for ( int i = (( m_Altitude + 1000 ) / 1000 ) * 1000 ; i < m_MaxHeight ; i += 1000 )
        {
        int y = middley - 20 - ( i - m_Altitude ) * height / m_MaxHeight ;
        dc.MoveTo ( rect1.left, y ) ;
        dc.LineTo ( rect1.left + 10, y ) ;
        }

    // draw the planes
    dc.SetTextColor ( RGB ( 255, 255, 255 )) ;

    // loop over the planes
    EuroScopePlugIn :: CAircraft    ac ;
    for ( ac = GetPlugIn () -> AircraftSelectFirst () ;
          ac.IsValid () ;
          ac = GetPlugIn () -> AircraftSelectNext ( ac ))
        {
        // ac position
        EuroScopePlugIn :: CAircraftPositionData    acpos = ac.GetPosition () ;

        // if too high (this is the cheapest check)
        if ( acpos.GetPressureAltitude () > m_MaxHeight )
            continue ;

        // calculate the distance to treshold
        double  dist = acpos.GetPosition ().DistanceTo ( m_RwyPos ) ;

        // if closer to 20
        if ( dist > 20 )
            continue ;

        // angle to the treshold
        double  angle = acpos.GetPosition ().DirectionTo ( m_RwyPos ) ;
        double  angle_diff = angle - m_Heading ;

        // if not acceptable
        if ( fabs ( angle_diff ) > 60 ||
             ( fabs ( angle_diff ) > 30 && dist > 5 ) ||
             ( fabs ( angle_diff ) > 15 && dist > 10 ) ||
             ( fabs ( angle_diff ) > 10 && dist > 15 ))
            continue ;

        // anglediff in radians
        angle_diff = angle_diff / 180.0 * M_PI ;

        // ok, we will display this aircraft
        // distance along the extended centerline
        double  rwy_dist = dist * cos ( angle_diff ) ;

        // distance from the extended centerline
        double  ctr_dist = dist * sin ( angle_diff ) ;

        // x coordinate in both views
        int x = (int) ( rect1.left + 5 + rwy_dist * width / 20.0 + 0.5 );

        // y coordinate in the slope
        int y1 = middley - 20 - ( acpos.GetPressureAltitude () - m_Altitude ) * height / m_MaxHeight ;

        // y coordinate in the extended centerline
        int y2 = (int) ( lovermiddley + width / 20.0 * ctr_dist + 0.5 ) ;

        // draw the plane in the slope
        // get the pitch
        int     pitch = ac.GetPosition ().GetReportedPitch () ;
        double  pitchcos ( cos ( pitch / 180.0 * M_PI )) ;
        double  pitchsin ( sin ( pitch / 180.0 * M_PI )) ;

        // plane symbol
        dc.SelectObject ( & pen3 ) ;
        dc.MoveTo ( x - (int) ( 30 * pitchcos + 0.5 ),
                    y1 + (int) ( 30 * pitchsin + 0.5 )) ;
        dc.LineTo ( x + (int) ( 30 * pitchcos + 0.5 ),
                    y1 - (int) ( 30 * pitchsin + 0.5 )) ;
        dc.LineTo ( x + (int) ( 30 * pitchcos - 15 * pitchsin + 0.5 ),
                    y1 - (int) ( 30 * pitchsin + 15 * pitchcos + 0.5 )) ;

        // middle point
        dc.SelectObject ( & pen4 ) ;
        dc.MoveTo ( x, y1 ) ;
        dc.LineTo ( x + 1, y1 ) ;

        // draw the plane in the extended line
        if ( y2 > middley + 10 && y2 < rect1.bottom - 5 )
            {
            // get the relative heading
            double  head = ac.GetPosition ().GetReportedHeading () - m_Heading ;
            double  headcos ( cos ( head / 180.0 * M_PI )) ;
            double  headsin ( sin ( head / 180.0 * M_PI )) ;

            // plane symbol
            dc.SelectObject ( & pen3 ) ;
            dc.MoveTo ( x - (int) ( 30 * headcos + 0.5 ),
                        y2 - (int) ( 30 * headsin + 0.5 )) ;
            dc.LineTo ( x + (int) ( 30 * headcos + 0.5 ),
                        y2 + (int) ( 30 * headsin + 0.5 )) ;
            dc.MoveTo ( x - (int) ( 10 * headcos - 20 * headsin + 0.5 ),
                        y2 - (int) ( 10 * headsin + 20 * headcos + 0.5 )) ;
            dc.LineTo ( x - (int) ( 10 * headcos + 21 * headsin + 0.5 ),
                        y2 - (int) ( 10 * headsin - 21 * headcos + 0.5 )) ;
            dc.MoveTo ( x - (int) ( - 20 * headcos - 5 * headsin + 0.5 ),
                        y2 - (int) ( -20  * headsin + 5 * headcos + 0.5 )) ;
            dc.LineTo ( x - (int) ( - 20 * headcos + 6 * headsin + 0.5 ),
                        y2 - (int) ( -20 * headsin - 6 * headcos + 0.5 )) ;

            // middle point
            dc.SelectObject ( & pen4 ) ;
            dc.MoveTo ( x, y2 ) ;
            dc.LineTo ( x + 1, y2 ) ;
            }

        // draw the callsign
        CString callsign = ac.GetCallsign () ;
        dc.TextOutA ( x, middley + 10, callsign ) ;
        }// loop over the planes

    // restore pen
    dc.SelectObject ( p_old_pen ) ;

    // detach
    dc.Detach () ;
}


//---OnCompileCommand---------------------------------------------------

bool    CPrecisionApproachScreen :: OnCompileCommand ( const char * sCommandLine )
{
    // only one command is accepted
    if ( strncmp ( sCommandLine, ".par", 4 ))
        return false ;

    // must use my own resources
    AFX_MANAGE_STATE ( AfxGetStaticModuleState ()) ;

    // fill in the settings dialog
    CPARSettingsDialog  dia ;
    dia.m_Description = m_Description ;
    dia.m_Longitude   = m_Longitude ;
    dia.m_Latitude    = m_Latitude ;
    dia.m_Altitude    = m_Altitude ;
    dia.m_Heading     = m_Heading ;
    dia.m_Slope       = m_Slope ;

    // open
    if ( dia.DoModal () != IDOK )
        return true ;

    // save the data back
    m_Description = dia.m_Description ;
    m_Longitude   = dia.m_Longitude ;
    m_Latitude    = dia.m_Latitude ;
    m_Altitude    = dia.m_Altitude ;
    m_Heading     = dia.m_Heading ;
    m_Slope       = dia.m_Slope ;

    // use the values
    m_RwyPos.LoadFromStrings ( m_Longitude, m_Latitude ) ;
    m_MaxHeight = m_Altitude + (int) ( 20.0 * 6076.0 * sin ( m_Slope / 180.0 * M_PI )) ;

    // we do not have to do anything as it will be updated in the next second

    return true ;
}
